<?php
/**
 * PART 2. Defining Custom Table List
 * ============================================================================
 *
 * In this part you are going to define custom table list class,
 * that will display your database records in nice looking table
 *
 * http://codex.wordpress.org/Class_Reference/WP_List_Table
 */
if(!class_exists('WP_List_Table'))
{
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

/**
 * Jss_sf_List_Table class that will display our custom table
 * records in nice table
 */
class Jss_sf_List_Table extends WP_List_Table
{

    /**
     * [REQUIRED] You must declare constructor and give some basic params
     */
    function __construct()
    {
        global $status, $page;

        parent::__construct(array(
            'singular' => 'banner',
            'plural' => 'banners',
        ));
    }

    /**
     * [REQUIRED] this is a default column renderer
     *
     * @param $item - row (key, value array)
     * @param $column_name - string (key)
     * @return HTML
     */
    function column_default($item, $column_name)
    {
        return $item[$column_name];
    }

    /**
     * [OPTIONAL] this is example, how to render specific column
     *
     * method name must be like this: "column_[column_name]"
     *
     * @param $item - row (key, value array)
     * @return HTML
     */
    function column_description($item)
    {
        return '<em>' . $item['description'] . '</em>';
    }

    function column_banner($item)
    {
        if(isset($item['banner']) && $item['banner'] != ''):
            return jss_sf_get_image_tag($item['banner']);
        else:
            return '';
        endif;
    }

    function column_shortcode($item)
    {
        return '[jss_sf_banners banner_id="' . $item['id'] . '"]';
    }

    /**
     * [OPTIONAL] this is example, how to render column with actions,
     * when you hover row "Edit | Delete" links showed
     *
     * @param $item - row (key, value array)
     * @return HTML
     */
    function column_title($item)
    {
        // links going to /admin.php?page=[your_plugin_page][&other_params]
        // notice how we used $_REQUEST['page'], so action will be done on curren page
        // also notice how we use $this->_args['singular'] so in this example it will
        // be something like &banner=2
        $actions = array(
            'edit' => sprintf('<a href="?page=banners_form&id=%s">%s</a>', $item['id'], __('Edit', 'jss_sf')),
            'delete' => sprintf('<a href="?page=%s&action=delete&id=%s">%s</a>', $_REQUEST['page'], $item['id'], __('Delete', 'jss_sf')),
        );

        return sprintf('%s %s', $item['title'], $this->row_actions($actions)
        );
    }

    /**
     * [REQUIRED] this is how checkbox column renders
     *
     * @param $item - row (key, value array)
     * @return HTML
     */
    function column_cb($item)
    {
        return sprintf(
                '<input type="checkbox" name="id[]" value="%s" />', $item['id']
        );
    }

    /**
     * [REQUIRED] This method return columns to display in table
     * you can skip columns that you do not want to show
     * like content, or description
     *
     * @return array
     */
    function get_columns()
    {
        $columns = array(
            'cb' => '<input type="checkbox" />', //Render a checkbox instead of text
            'title' => __('Title', 'jss_sf'),
            'banner' => __('Banner', 'jss_sf'),
            'description' => __('Description', 'jss_sf')
        );
        return $columns;
    }

    /**
     * [OPTIONAL] This method return columns that may be used to sort table
     * all strings in array - is column names
     * notice that true on name column means that its default sort
     *
     * @return array
     */
    function get_sortable_columns()
    {
        $sortable_columns = array(
            'title' => array('title', true)
        );
        return $sortable_columns;
    }

    /**
     * [OPTIONAL] Return array of bult actions if has any
     *
     * @return array
     */
    function get_bulk_actions()
    {
        $actions = array(
            'delete' => 'Delete'
        );
        return $actions;
    }

    /**
     * [OPTIONAL] This method processes bulk actions
     * it can be outside of class
     * it can not use wp_redirect coz there is output already
     * in this example we are processing delete action
     * message about successful deletion will be shown on page in next part
     */
    function process_bulk_action()
    {
        global $wpdb;
        $table_name_banners = $wpdb->prefix . 'jss_sf_banners'; // do not forget about tables prefix

        if('delete' === $this->current_action())
        {
            $ids = isset($_REQUEST['id']) ? $_REQUEST['id'] : array();
            if(is_array($ids))
                $ids = implode(',', $ids);

            if(!empty($ids))
            {
                $wpdb->query("DELETE FROM $table_name_banners WHERE id IN($ids)");
            }
        }
    }

    /**
     * [REQUIRED] This is the most important method
     *
     * It will get rows from database and prepare them to be showed in table
     */
    function prepare_items()
    {
        global $wpdb;
        $table_name_banners = $wpdb->prefix . 'jss_sf_banners'; // do not forget about tables prefix

        $per_page = 5; // constant, how much records will be shown per page

        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();

        // here we configure table headers, defined in our methods
        $this->_column_headers = array($columns, $hidden, $sortable);

        // [OPTIONAL] process bulk action if any
        $this->process_bulk_action();

        // will be used in pagination settings
        $total_items = $wpdb->get_var("SELECT COUNT(id) FROM $table_name_banners");

        // prepare query params, as usual current page, order by and order direction
        $paged = isset($_REQUEST['paged']) ? max(0, intval($_REQUEST['paged']) - 1) : 0;
        $orderby = (isset($_REQUEST['orderby']) && in_array($_REQUEST['orderby'], array_keys($this->get_sortable_columns()))) ? $_REQUEST['orderby'] : 'title';
        $order = (isset($_REQUEST['order']) && in_array($_REQUEST['order'], array('asc', 'desc'))) ? $_REQUEST['order'] : 'asc';

        // [REQUIRED] define $items array
        // notice that last argument is ARRAY_A, so we will retrieve array
        $this->items = $wpdb->get_results($wpdb->prepare("SELECT * FROM $table_name_banners ORDER BY $orderby $order LIMIT %d OFFSET %d", $per_page, $paged), ARRAY_A);

        // [REQUIRED] configure pagination
        $this->set_pagination_args(array(
            'total_items' => $total_items, // total items defined above
            'per_page' => $per_page, // per page constant defined at top of method
            'total_pages' => ceil($total_items / $per_page) // calculate pages count
        ));
    }

}

/**
 * List page handler
 *
 * This function renders our custom table
 * Notice how we display message about successfull deletion
 * Actualy this is very easy, and you can add as many features
 * as you want.
 *
 * Look into /wp-admin/includes/class-wp-*-list-table.php for examples
 */
function jss_sf_banners_page_handler()
{
    global $wpdb;

    $table = new Jss_sf_List_Table();
    $table->prepare_items();

    $message = '';
    if('delete' === $table->current_action())
    {
        $message = '<div class="updated below-h2" id="message"><p>' . sprintf(__('Items deleted: %d', 'jss_sf'), count($_REQUEST['id'])) . '</p></div>';
    }
    ?>
    <div class="wrap">

        <div class="icon32 icon32-posts-post" id="icon-edit"><br></div>
        <h2><?php _e('Jss_sf', 'jss_sf') ?> <a class="add-new-h2"
                                               href="<?php echo get_admin_url(get_current_blog_id(), 'admin.php?page=banners_form'); ?>"><?php _e('Add New', 'jss_sf') ?></a>
        </h2>
        <?php echo $message; ?>

        <form id="persons-table" method="GET">
            <input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>"/>
            <?php $table->display() ?>
        </form>

    </div>
    <?php
}

/**
 * PART 4. Form for adding andor editing row
 * ============================================================================
 *
 * In this part you are going to add admin page for adding andor editing items
 * You cant put all form into this function, but in this example form will
 * be placed into meta box, and if you want you can split your form into
 * as many meta boxes as you want
 *
 * http://codex.wordpress.org/Data_Validation
 * http://codex.wordpress.org/Function_Reference/selected
 */

/**
 * Form page handler checks is there some data posted and tries to save it
 * Also it renders basic wrapper in which we are callin meta box render
 */
function jss_sf_banners_form_page_handler()
{
    global $wpdb;
    $table_name_banners = $wpdb->prefix . 'jss_sf_banners'; // do not forget about tables prefix

    $message = '';
    $notice = '';

    // this is default $item which will be used for new records
    $default = array(
        'id' => 0,
        'title' => '',
        'description' => '',
        'created_at' => date('Y-m-d H:i:s')
    );

    // here we are verifying does this request is post back and have correct nonce
    if(wp_verify_nonce($_REQUEST['nonce'], basename(__FILE__)))
    {
        // combine our default item with request params
        //print_r($_FILES);
        $item = shortcode_atts($default, $_REQUEST);
        if($_FILES['banner']['name'] != '')
        {
            $item['banner'] = jss_sf_fileupload_process();
        }
        // validate data, and if all ok save item to database
        // if id is zero insert otherwise update
        $item_valid = jss_sf_validate_banner($item);
        if($item_valid === true)
        {
            if($item['id'] == 0)
            {
                $result = $wpdb->insert($table_name_banners, $item);
                $item['id'] = $wpdb->insert_id;
                if($result)
                {
                    $message = __('Item was successfully saved', 'jss_sf');
                }
                else
                {
                    $notice = __('There was an error while saving item', 'jss_sf');
                }
            }
            else
            {
                if($_FILES['banner']['name'] == '')
                {
                    $item['banner'] = $_POST['prev_banner'];
                }
                $result = $wpdb->update($table_name_banners, $item, array('id' => $item['id']));
                if($result)
                {

                    $message = __('Item was successfully updated', 'jss_sf');
                }
                else
                {
                    $notice = __('There was an error while updating item', 'jss_sf');
                }
            }
        }
        else
        {
            // if $item_valid not true it contains error message(s)
            $notice = $item_valid;
        }
    }
    else
    {
        // if this is not post back we load item to edit or give new one to create
        $item = $default;
        if(isset($_REQUEST['id']))
        {
            $item = $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name_banners WHERE id = %d", $_REQUEST['id']), ARRAY_A);
            if(!$item)
            {
                $item = $default;
                $notice = __('Item not found', 'jss_sf');
            }
        }
    }

    // here we adding our custom meta box
    add_meta_box('banners_form_meta_box', 'Banner data', 'jss_sf_banners_form_meta_box_handler', 'banner', 'normal', 'default');
    ?>
    <div class="wrap">
        <div class="icon32 icon32-posts-post" id="icon-edit"><br></div>
        <h2><?php _e('Banner', 'jss_sf') ?> <a class="add-new-h2"
                                               href="<?php echo get_admin_url(get_current_blog_id(), 'admin.php?page=banners'); ?>"><?php _e('Back to List', 'jss_sf') ?></a>
        </h2>

        <?php if(!empty($notice)): ?>
            <div id="notice" class="error"><p><?php echo $notice ?></p></div>
        <?php endif; ?>
        <?php if(!empty($message)): ?>
            <div id="message" class="updated"><p><?php echo $message ?></p></div>
        <?php endif; ?>

        <form id="form" method="POST" enctype="multipart/form-data">
            <input type="hidden" name="nonce" value="<?php echo wp_create_nonce(basename(__FILE__)) ?>"/>
            <?php /* NOTICE: here we storing id to determine will be item added or updated */ ?>
            <input type="hidden" name="id" value="<?php echo $item['id'] ?>"/>

            <div class="metabox-holder" id="poststuff">
                <div id="post-body">
                    <div id="post-body-content">
                        <?php /* And here we call our custom meta box */ ?>
                        <?php do_meta_boxes('banner', 'normal', $item); ?>
                        <input type="submit" value="<?php _e('Save', 'jss_sf') ?>" id="submit" class="button-primary" name="submit">
                    </div>
                </div>
            </div>
        </form>
    </div>
    <?php
}

/**
 * This function renders our custom meta box
 * $item is row
 *
 * @param $item
 */
function jss_sf_banners_form_meta_box_handler($item)
{
    ?>

    <table cellspacing="2" cellpadding="5" style="width: 100%;" class="form-table">
        <tbody>
            <tr class="form-field">
                <th valign="top" scope="row">
                    <label for="title"><?php _e('Title', 'jss_sf') ?></label>
                </th>
                <td>
                    <input id="name" name="title" type="text" style="width: 50%" value="<?php echo esc_attr($item['title']) ?>"
                           size="50" class="code" placeholder="<?php _e('Banner Title', 'jss_sf') ?>" required>
                </td>
            </tr>
            <tr class="form-field">
                <th valign="top" scope="row">
                    <label for="description"><?php _e('Description', 'jss_sf') ?></label>
                </th>
                <td>
                    <textarea id="description" name="description" cols="50" rows="10" ><?php echo esc_attr($item['description']) ?></textarea>
                    <!--<input id="description" name="description" type="text" style="width: 95%" value="<?php echo esc_attr($item['description']) ?>"                           size="50" class="code" placeholder="<?php _e('Banner Description', 'jss_sf') ?>" required>-->
                </td>
            </tr>
            <tr class="form-field">
                <th valign="top" scope="row">
                    <label for="banner"><?php _e('Banner', 'jss_sf') ?></label>
                </th>
                <td>
                    <?php
                    if(isset($item['banner']) && $item['banner'] != ''):
                        echo jss_sf_get_image_tag($item['banner']);
                        $image_required = '';
                        $prev_banner = $item['banner'];
                    else:
                        $image_required = 'required';
                        $prev_banner = '';
                    endif;
                    ?>
                    <input type="hidden" name="prev_banner" value="<?php echo $prev_banner; ?>" />
                    <input id="banner" name="banner" type="file" style="width: 95%"
                           size="50" class="code" placeholder="<?php _e('Banner', 'jss_sf') ?>" <?php echo $image_required; ?>>
                </td>
            </tr>
        </tbody>
    </table>
    <?php
}

/**
 * Simple function that validates data and retrieve bool on success
 * and error message(s) on error
 *
 * @param $item
 * @return bool|string
 */
function jss_sf_validate_banner($item)
{
    $messages = array();

    if(empty($item['title']))
        $messages[] = __('Title is required', 'jss_sf');
    //if(!empty($item['age']) && !absint(intval($item['age'])))  $messages[] = __('Age can not be less than zero');
    //if(!empty($item['age']) && !preg_match('/[0-9]+/', $item['age'])) $messages[] = __('Age must be number');
    //...

    if(empty($messages))
        return true;
    return implode('<br />', $messages);
}

/**
 * Handle file uploads
 *
 * @todo check nonces
 * @todo check file size
 *
 * @return none
 */
function jss_sf_fileupload_process()
{

    $image_name = '';

    if($_FILES['banner']['name'] != '')
    {
        $upload_dir = wp_upload_dir();

        if(!is_writeable($upload_dir['path']))
        {
            echo 'Unable to write to directory %s. Is this directory writable by the server?';
            //$this->msg_e('Unable to write to directory %s. Is this directory writable by the server?');
            return;
        }

        $upload_array['files_array'] = $_FILES['banner'];
        $upload_array['image_prefix'] = uniqid() . '_';
        $upload_array['upload_dir'] = $upload_dir['basedir'];

        require_once 'lib/banner_upload.php';
        $banner_upload = new Banner_upload();

        $image_name = $banner_upload->process($upload_array);
    }
    return $image_name;
}
?>